// MazeSolution.java
//
// ICS 23 Project #1: Dark at the End of the Tunnel
// Coded by Alex Thornton Fall 2002
// Minor edits by Norman Jacobson January 2003


// The MazeSolution interface defines the behavior of a maze solution.
// A maze solution is a sequence of positions (i.e. (x,y) coordinates in the
// maze grid, where (0,0) is the top left corner of the maze).  A just-created
// maze solution contains the start position. Positions are added to the sequence
// by "moving" into a new cell solution; they can be removed by "backing it out" of
// a cell.
//
// Several of these methods can throw the unchecked exception MazeSolutionException; we
// designed the methods this way so your program will complain if it is asked to make an
// illegal move when building or solving a maze. Thus, if this exception is thrown, your
// program has a bug that needs to be fixed!
//
// You create a MazeSolution by calling this static method:
//
//     MazeSolution MazeSolutionFactory.createMazeSolution(
//         int width, int height,
//         int startXPos, int startYPos,
//         int endXPos, int endYPos)
//
// The six parameters describe the maze for which you are
// searching for a solution: its width and height,
// the x- and y-coordinates of the starting position of
// the maze, and the x- and y-coordinates of the ending position
// of the maze.
//
// NOTE: You may not need to use all of the methods in this interface.

public interface MazeSolution
{
	// getMazeWidth() returns the width of the maze for which this is a
	// solution.
	public int getMazeWidth();

	// getMazeHeight() returns the height of the maze for which this is a
	// solution.
	public int getMazeHeight();

	// solutionComplete() returns true if the last position in the sequence
	// is the ending position of the maze, or false if not.
	public boolean solutionComplete();

	// getLastXPosition() returns the x-coordinate of the last position in
	// the sequence.
	public int getLastXPosition();

	// getLastYPosition() returns the y-coordinate of the last position in
	// the sequence.
	public int getLastYPosition();

	// getXPositions() returns an array of the x-coordinates of all of the
	// positions in the sequence.  The 0th cell of the array contains the
	// x-coordinate of the first position in the sequence, and so on.  The
	// length of the array will be equal to the number of positions in
	// the sequence.
	public int[] getXPositions();

	// getYPositions() returns an array of the y-coordinates of all of the
	// positions in the sequence.  The 0th cell of the array contains the
	// y-coordinate of the first position in the sequence, and so on.  The
	// length of the array will be equal to the number of positions in
	// the sequence.
	public int[] getYPositions();

	// getLength() returns the number of positions in the sequence.
	public int getLength();

	// moveNorth() adds a new position to the sequence that is one cell above
	// the last position.  Throws a MazeSolutionException if the last position
	// is in top row of the maze.
	public void moveNorth();

	// moveSouth() adds a new position to the sequence that is one cell below
	// the last position.  Throws a MazeSolutionException if the last position
	// is in the bottom row of the maze.
	public void moveSouth();

	// moveEast() adds a new position to the sequence that is one cell to the
	// right of the last position.  Throws a MazeSolutionException if the last
	// position is in the rightmost column of the maze.
	public void moveEast();

	// moveWest() adds a new position to the sequence that is one cell to the
	// left of the last position.  Throws a MazeSolutionException if the last
	// position is in the leftmost column of the maze.
	public void moveWest();

	// backUp() removes the last position from the sequence.  Throws a
	// MazeSolutionException if there is only one position in the sequence.
	public void backUp();
}
